@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2017
+# Copyright (C) 2015-2018
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -16,9 +16,9 @@
 #
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
-"""This module contains the classes that represent Telegram InlineQueryResultCachedSticker"""
+"""This module contains the classes that represent Telegram InlineQueryResultCachedSticker."""
 
-from telegram import InlineQueryResult, InlineKeyboardMarkup, InputMessageContent
+from telegram import InlineQueryResult
 
 
 class InlineQueryResultCachedSticker(InlineQueryResult):
@@ -44,6 +44,7 @@ class InlineQueryResultCachedSticker(InlineQueryResult):
         input_message_content (:class:`telegram.InputMessageContent`, optional): Content of the
             message to be sent instead of the sticker.
         **kwargs (:obj:`dict`): Arbitrary keyword arguments.
+
     """
 
     def __init__(self,
@@ -57,20 +58,5 @@ class InlineQueryResultCachedSticker(InlineQueryResult):
         self.sticker_file_id = sticker_file_id
 
         # Optionals
-        if reply_markup:
-            self.reply_markup = reply_markup
-        if input_message_content:
-            self.input_message_content = input_message_content
-
-    @classmethod
-    def de_json(cls, data, bot):
-        data = super(InlineQueryResultCachedSticker, cls).de_json(data, bot)
-
-        if not data:
-            return None
-
-        data['reply_markup'] = InlineKeyboardMarkup.de_json(data.get('reply_markup'), bot)
-        data['input_message_content'] = InputMessageContent.de_json(
-            data.get('input_message_content'), bot)
-
-        return cls(**data)
+        self.reply_markup = reply_markup
+        self.input_message_content = input_message_content
