@@ -17,11 +17,12 @@
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 """This module contains methods to make POST and GET requests."""
+import logging
 import os
 import socket
 import sys
-import logging
 import warnings
+from builtins import str  # For PY2
 
 try:
     import ujson as json
@@ -35,17 +36,34 @@ try:
     import telegram.vendor.ptb_urllib3.urllib3.contrib.appengine as appengine
     from telegram.vendor.ptb_urllib3.urllib3.connection import HTTPConnection
     from telegram.vendor.ptb_urllib3.urllib3.util.timeout import Timeout
+    from telegram.vendor.ptb_urllib3.urllib3.fields import RequestField
 except ImportError:  # pragma: no cover
     warnings.warn("python-telegram-bot wasn't properly installed. Please refer to README.rst on "
                   "how to properly install.")
     raise
 
-from telegram import (InputFile, TelegramError)
+from telegram import (InputFile, TelegramError, InputMedia)
 from telegram.error import (Unauthorized, NetworkError, TimedOut, BadRequest, ChatMigrated,
-                            RetryAfter, InvalidToken)
+                            RetryAfter, InvalidToken, Conflict)
+
+
+def _render_part(self, name, value):
+    """
+    Monkey patch urllib3.urllib3.fields.RequestField to make it *not* support RFC2231 compliant
+    Content-Disposition headers since telegram servers don't understand it. Instead just escape
+    \ and " and replace any \n and \r with a space.
+    """
+    value = value.replace(u'\\', u'\\\\').replace(u'"', u'\\"')
+    value = value.replace(u'\r', u' ').replace(u'\n', u' ')
+    return u'%s="%s"' % (name, value)
+
+
+RequestField._render_part = _render_part
 
 logging.getLogger('urllib3').setLevel(logging.WARNING)
 
+USER_AGENT = 'Python Telegram Bot (https://github.com/python-telegram-bot/python-telegram-bot)'
+
 
 class Request(object):
     """
@@ -83,9 +101,12 @@ class Request(object):
 
         # TODO: Support other platforms like mac and windows.
         if 'linux' in sys.platform:
-            sockopts.append((socket.IPPROTO_TCP, socket.TCP_KEEPIDLE, 120))
-            sockopts.append((socket.IPPROTO_TCP, socket.TCP_KEEPINTVL, 30))
-            sockopts.append((socket.IPPROTO_TCP, socket.TCP_KEEPCNT, 8))
+            sockopts.append((socket.IPPROTO_TCP,
+                             socket.TCP_KEEPIDLE, 120))  # pylint: disable=no-member
+            sockopts.append((socket.IPPROTO_TCP,
+                             socket.TCP_KEEPINTVL, 30))  # pylint: disable=no-member
+            sockopts.append((socket.IPPROTO_TCP,
+                             socket.TCP_KEEPCNT, 8))  # pylint: disable=no-member
 
         self._con_pool_size = con_pool_size
 
@@ -190,6 +211,8 @@ class Request(object):
         if 'headers' not in kwargs:
             kwargs['headers'] = {}
         kwargs['headers']['connection'] = 'keep-alive'
+        # Also set our user agent
+        kwargs['headers']['user-agent'] = USER_AGENT
 
         try:
             resp = self._con_pool.request(*args, **kwargs)
@@ -215,6 +238,8 @@ class Request(object):
             raise BadRequest(message)
         elif resp.status == 404:
             raise InvalidToken()
+        elif resp.status == 409:
+            raise Conflict(message)
         elif resp.status == 413:
             raise NetworkError('File too large. Check telegram api limits '
                                'https://core.telegram.org/bots/api#senddocument')
@@ -264,18 +289,42 @@ class Request(object):
         if timeout is not None:
             urlopen_kwargs['timeout'] = Timeout(read=timeout, connect=self._connect_timeout)
 
-        if InputFile.is_inputfile(data):
-            data = InputFile(data)
-            result = self._request_wrapper(
-                'POST', url, body=data.to_form(), headers=data.headers, **urlopen_kwargs)
+        # Are we uploading files?
+        files = False
+
+        for key, val in data.copy().items():
+            if isinstance(val, InputFile):
+                # Convert the InputFile to urllib3 field format
+                data[key] = val.field_tuple
+                files = True
+            elif isinstance(val, (float, int)):
+                # Urllib3 doesn't like floats it seems
+                data[key] = str(val)
+            elif key == 'media':
+                # One media or multiple
+                if isinstance(val, InputMedia):
+                    # Attach and set val to attached name
+                    data[key] = val.to_json()
+                    if isinstance(val.media, InputFile):
+                        data[val.media.attach] = val.media.field_tuple
+                else:
+                    # Attach and set val to attached name for all
+                    media = []
+                    for m in val:
+                        media.append(m.to_dict())
+                        if isinstance(m.media, InputFile):
+                            data[m.media.attach] = m.media.field_tuple
+                    data[key] = json.dumps(media)
+                files = True
+
+        # Use multipart upload if we're uploading files, otherwise use JSON
+        if files:
+            result = self._request_wrapper('POST', url, fields=data, **urlopen_kwargs)
         else:
-            data = json.dumps(data)
-            result = self._request_wrapper(
-                'POST',
-                url,
-                body=data.encode(),
-                headers={'Content-Type': 'application/json'},
-                **urlopen_kwargs)
+            result = self._request_wrapper('POST', url,
+                                           body=json.dumps(data).encode('utf-8'),
+                                           headers={'Content-Type': 'application/json'},
+                                           **urlopen_kwargs)
 
         return self._parse(result)
 
